# -*- encoding: utf-8 -*-
"""
@文件        :spider.py
@说明        :
@时间        :2024/11/25 20:01:51
@作者        :Zack
@版本        :1.0
"""
from dataclasses import dataclass

from gmscaffold.base.dataclass import BaseData


@dataclass
class SpiderData(BaseData):
    """ """



# -*- encoding: utf-8 -*-
"""
@文件        :test_signal.py
@说明        :
@时间        :2024/11/25 11:45:30
@作者        :Zack
@版本        :1.0
"""
import unittest

from pydispatch import dispatcher


def handle_print1(sender, **kwargs):
    """测试函数"""
    print("[*] handle_print1:", sender, kwargs)


def handle_print2(sender, **kwargs):
    """测试函数"""
    print("[*] handle_print2:", sender, kwargs)


signal_print1 = object()
signal_print2 = object()


class TestSignal(unittest.TestCase):
    """ """

    def test_signal(self):
        """ """
        dispatcher.connect(handle_print1, signal_print1)
        dispatcher.connect(handle_print2, signal_print2)
        dispatcher.send(signal=signal_print1, sender=dispatcher.Anonymous, item="hello print")
        dispatcher.send(signal=signal_print1, sender=dispatcher.Anonymous, item="hello print")
        dispatcher.send(signal=signal_print2, sender=dispatcher.Anonymous, item="hello print")


if __name__ == "__main__":
    unittest.main()



# -*- encoding: utf-8 -*-
"""
@文件        :decorators.py
@说明        :
@时间        :2024/11/28 11:57:30
@作者        :Zack
@版本        :1.0
"""
import sys
from functools import partial, wraps


def hide_except(fn=None, exc_type: BaseException = None):
    """ """
    if fn is None:
        return partial(hide_except, exc_type=exc_type)

    @wraps(fn)
    def inner(*args, **kwargs):
        """ """
        try:
            return fn(*args, **kwargs)
        except exc_type as e:
            print("\n [gm] 退出...")
            sys.exit()

    return inner



# -*- encoding: utf-8 -*-
"""
@文件        :WebCommand.py
@说明        :
@时间        :2024/11/27 18:44:40
@作者        :Zack
@版本        :1.0
"""

from typing import Any, Dict, Optional, TypeVar

from gmscaffold.base.command import BaseCommand
from gmscaffold.entities.web import WebData
from gmscaffold.utils import signal as _signal

ProjectObject = TypeVar("ProjectObject", bound=WebData)


class AppCommand(object):
    """ """

    def _input_app_name(self, project_object: ProjectObject):
        """ """
        project_name = input("请输入应用名称: ")
        project_object.name = project_name
        return project_object


class CreateWebCommand(BaseCommand):
    """ """

    name = "web start_project"
    description = "Creating a traditional web service based on the sanic framework"

    def start_project(self, *args, **kwargs):
        """Create sanic web server project"""
        web_data = WebData()
        self._input_project_name(web_data)._input_project_author(web_data)._input_project_email(
            web_data
        )._input_project_version(web_data)._input_project_desc(web_data)._input_project_date(web_data)
        self.Gm._signals.send(signal=_signal.create_web_project, data=web_data)

    def handle(self):
        self.start_project()


class RemoveWebCommand(BaseCommand):
    """ """

    name = "web remove_project"
    description = "Remove the traditional web service based on scaffolding to create sanic framework"

    def remove_project(self, *args, **kwargs):
        """ """
        web_data = WebData()
        # TODO: input remove project name
        self.Gm._signals.send(signal=_signal.remove_web_project)

    def handle(self):
        self.remove_project()


class StartAppWebCommand(BaseCommand):
    """ """

    name = "web start_app"
    description = "Create a web service internal sub-application"

    def start_app(self, *args, **kwargs):
        """Create sanic web app"""
        web_data = WebData()
        AppCommand()._input_app_name(web_data)
        self.Gm._signals.send(signal=_signal.create_web_app, data=web_data)

    def handle(self):
        self.start_app()


class RemoveAppWebCommand(BaseCommand):
    """ """

    name = "web remove_app"
    description = "Remove the creation of internal sub-applications of web services based on scaffolding"

    def remove_app(self, *args, **kwargs):
        """Remove sanic web app"""
        web_data = WebData()
        # TODO: input remove app name
        self.Gm._signals.send(signal=_signal.remove_web_app)

    def handle(self):
        self.remove_app()


class WebCommand(BaseCommand):
    """ """

    name = "web"
    description = "Web service scaffolding based on Sanic"
    commands = [
        CreateWebCommand,
        RemoveWebCommand,
        StartAppWebCommand,
        RemoveAppWebCommand,
    ]



# -*- encoding: utf-8 -*-
"""
@文件        :reflect.py
@说明        :
@时间        :2024/11/25 11:13:26
@作者        :Zack
@版本        :1.0
"""

from typing import Callable, Type, TypeVar

_inlineCallbacksExtraneous = []


_T_Callable = TypeVar("_T_Callable", bound=Callable[..., object])


def qual(clazz: Type[object]) -> str:
    """
    Return full import path of a class.
    """
    return clazz.__module__ + "." + clazz.__name__


def _extraneous(f: _T_Callable) -> _T_Callable:
    """
    Mark the given callable as extraneous to inlineCallbacks exception
    reporting; don't show these functions.

    @param f: a function that you NEVER WANT TO SEE AGAIN in ANY TRACEBACK
        reported by Failure.

    @type f: function

    @return: f
    """
    _inlineCallbacksExtraneous.append(f.__code__)
    return f


def declspec_check(inst, declspec: str):
    """Calling specification, print the calling specifications supported by the current object

    :param inst: Reflection Objects
    :param declspec: Reflection Method
    """
    inst_declspec = dir(inst)
    dir_declspec_atts = "\n\t".join(filter(lambda x: not x.startswith("_"), inst_declspec))

    # TODO: 使用logger
    print(f"ERROR : Could not consume arg: {declspec}")
    print("Usage <command>\n    available commands:")
    print("\t{:>10}".format(dir_declspec_atts))


def declspec_revoke(inst, declspec: str, *args, **kwargs):
    """Call

    :param inst: Reflection Objects
    :param declspec: Reflection Method
    :param args: Reflecting method parameters
    :param kwargs: Reflecting method parameters
    """
    if hasattr(inst, declspec):
        method = getattr(inst, declspec)
        if callable(method):
            method(*args, **kwargs)
    else:
        declspec_check(inst, declspec)



# -*- encoding: utf-8 -*-
"""
@文件        :spider.py
@说明        :
@时间        :2024/11/25 20:01:51
@作者        :Zack
@版本        :1.0
"""
import datetime
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


@dataclass
class BaseData:
    """ """

    #: 包名
    package: Optional[str] = field(default="")
    # 项目名称
    name: Optional[str] = field(default="")
    # 作者
    author: Optional[str] = field(default="")
    # 邮箱
    email: Optional[str] = field(default="")
    # 版本
    version: Optional[str] = field(default="")
    # 描述信息
    description: Optional[str] = field(default="")
    # 创建时间
    date: Optional[str] = field(default=datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S"))


@dataclass
class TemplateData:
    """ """

    files: List[str]
    render: Dict[str, Any]
    template_path: str
    template_path_root: str
    target_dir: Optional[str] = field(default=None)
    app: str = False



# -*- encoding: utf-8 -*-
"""
@文件        :SpiderCommands.py
@说明        :
@时间        :2024/11/25 19:51:23
@作者        :Zack
@版本        :1.0
"""

from gmscaffold.base.command import BaseCommand
from gmscaffold.entities.spider import SpiderData
from gmscaffold.utils import signal as _signal


class CreateSpiderCommand(BaseCommand):
    """ """

    name = "spider create_spider"
    description = "Crawler script creation"

    def create_spider(self, *args, **kwargs):
        """Create spider scripts"""
        spider_data = SpiderData()
        self._input_project_name(spider_data)._input_project_author(
            spider_data
        )._input_project_email(spider_data)._input_project_version(
            spider_data
        )._input_project_desc(
            spider_data
        )._input_project_date(
            spider_data
        )
        self.Gm._signals.send(signal=_signal.create_spider_project, data=spider_data)

    def handle(self):
        self.create_spider()


class RemoveSpiderCommand(BaseCommand):

    name = "spider remove_spider"
    description = "Crawler script removal"

    def remove_spider(self, spider_name: str):
        """Remove spider name

        :param spider_name: Project name
        """
        print("remove_spider ", spider_name)

    def handle(self):
        self.remove_spider()


class SpiderCommand(BaseCommand):
    """ """

    name = "spider"
    description = "Crawler script project scaffolding"
    commands = [CreateSpiderCommand, RemoveSpiderCommand]



# -*- encoding: utf-8 -*-
"""
@文件        :template.py
@说明        :
@时间        :2024/11/27 11:43:41
@作者        :Zack
@版本        :1.0
"""
import os
import sys
from pathlib import Path, PurePosixPath, PureWindowsPath
from typing import Any, Dict

import jinja2

from gmscaffold.base.dataclass import TemplateData
from gmscaffold.exceptions import BuildValueError
from gmscaffold.utils import path as _path


def to_camel_case(s):
    """ """
    return "".join(word.title() for word in s.split("_"))


def to_capitalize(s):
    """ """
    return s.capitalize()


def to_upper(s):
    """ """
    return s.upper()


class Template:
    """Generate project folder"""

    def __init__(self, template_path: str, render: Dict[str, Any], target_dir: str = None):
        """
        :param template_path : Project template path
        :param target_dir : Create folder into target directory
        """
        self._target_dir = target_dir or os.getcwd()
        self._template_path = template_path

        if not isinstance(render, dict):
            """ """
            raise BuildValueError(f"参数类型错误,期望类型: dict; {render}")

        self._render_vars = render
        os.makedirs(self._target_dir, exist_ok=True)

    def exits(self) -> bool:
        """ """
        return os.path.exists(self._template_path)

    def basename(self, path: str) -> str:
        """ """
        _path = Path(path)
        return _path.name

    def to_cwd(self, basename: str):
        """ """
        if "win" in sys.platform:
            pure_path = PureWindowsPath(self._target_dir)
        else:
            pure_path = PurePosixPath(self._target_dir)
        return pure_path.with_name(basename)

    def iter_listdir(self):
        """ """
        if self.exits():
            yield from os.listdir(self._template_path)

    def listdir(self):
        """ """
        if not self.exits():
            return []
        return os.listdir(self._template_path)

    def render(self, file: str):
        """ """
        path = Path(file)
        template_path = os.path.dirname(file)
        template_loader = jinja2.FileSystemLoader(searchpath=template_path)
        template_env = jinja2.Environment(loader=template_loader)
        template_env.filters["to_capitalize"] = to_capitalize
        template = template_env.get_template(path.name)
        rendered_code = template.render(self._render_vars)
        return rendered_code

    def fwrite(self, content: str, file: str, mode="w", encoding="utf-8"):
        """
        :param content: 文件内容
        :param file: 需要写入的文件
        """
        file_folder = os.path.dirname(file)
        new_file = _path.replace_suffix(file, old=".tmpl", new=".py")
        os.makedirs(file_folder, exist_ok=True)
        with open(new_file, mode, encoding=encoding) as f:
            f.write(content)


def build_project(tpl_data: TemplateData):
    """ """
    tpl = Template(tpl_data.template_path, tpl_data.render, tpl_data.target_dir)
    project_folder = os.path.join(tpl._target_dir, tpl_data.template_path_root)
    os.makedirs(project_folder, exist_ok=True)
    for file in tpl_data.files:
        """ """
        rendered_code = tpl.render(file)
        n_path = _path.replace_path(file, tpl_data.template_path, project_folder)
        tpl.fwrite(rendered_code, n_path)


def build_app(tpl_data: TemplateData):
    """ """
    #: 构建
    tpl = Template(tpl_data.template_path, tpl_data.render, tpl_data.target_dir)
    os.makedirs(tpl_data.template_path_root, exist_ok=True)
    for file in tpl_data.files:
        rendered_code = tpl.render(file)
        n_path = _path.replace_path(file, tpl_data.template_path, tpl_data.template_path_root)
        tpl.fwrite(rendered_code, n_path)



# -*- encoding: utf-8 -*-
"""
@文件        :builder.py
@说明        :
@时间        :2025/01/16 10:43:38
@作者        :Zack
@版本        :1.0
"""
import os
from dataclasses import asdict

from gmscaffold.base.dataclass import BaseData, TemplateData
from gmscaffold.utils import path as _path
from gmscaffold.utils.conf import get_module_basename_with_path, get_module_name

APP_NAME = "app"
APP_SKIP_FOLDER = [APP_NAME]


def make_template_data(ptr, pData: BaseData):
    """ """
    module_name = get_module_name(ptr)
    module_tpl = ptr.Gm._settings.extension_template(module_name)
    files_list = _path.get_files_in_directory(module_tpl, ignore_folders=APP_SKIP_FOLDER)
    tpl_root = pData.name
    project_data = asdict(pData)
    tpl_data = TemplateData(files_list, project_data, module_tpl, tpl_root)
    return tpl_data


def make_template_app(ptr, pData: BaseData):
    """ """
    cwd = os.getcwd()
    tpl_root = os.path.join(cwd, "blueprints", pData.name)
    _module_name = get_module_name(ptr)
    _module_tpl = ptr.Gm._settings.extension_template(_module_name)
    module_tpl = os.path.join(_module_tpl, APP_NAME)
    files_list = _path.get_files_in_directory(module_tpl)
    project_data = asdict(pData)
    project_data["package"] = get_module_basename_with_path(cwd)
    tpl_data = TemplateData(files_list, project_data, module_tpl, tpl_root)
    return tpl_data



# -*- encoding: utf-8 -*-
"""
@文件        :signal.py
@说明        :
@时间        :2024/11/25 11:35:42
@作者        :Zack
@版本        :1.0
"""

create_spider_project = object()
remove_spider_project = object()

create_web_project = object()
create_web_app = object()
remove_web_project = object()
remove_web_app = object()

create_gmapp_project = object()
remove_gmapp_project = object()

OPEN_CONNECT = 1
CLOSE_CONNECT = 0



# -*- encoding: utf-8 -*-
"""
@文件        :cmdline.py
@说明        :
@时间        :2024/11/25 10:54:59
@作者        :Zack
@版本        :1.0
"""
import enum


class ProjectEnum(enum.Enum):
    """Project type enumeration class,
        used to specify different styles of creating projects.

    The following are the available project types:
    - SPIDER: represents the style of spider script projects, suitable for [script projects of specific applicable script types].
    - SCRAPY: represents the style of spider Scrapy projects, used for [specifically applicable Scrapy script projects].
    - GM_APP: represents the style of GM (App) projects, often used for [specifically applicable GMB-APP application projects].
    - WEB_SERVER: represents the style of traditional Web projects, often used for [specifically using traditional Web projects].
    """

    SPIDER = enum.auto()
    SCRAPY = enum.auto()
    GM_APP = enum.auto()
    WEB_SERVER = enum.auto()



# -*- encoding: utf-8 -*-
"""
@文件        :test_cmdline.py
@说明        :
@时间        :2024/11/26 15:40:03
@作者        :Zack
@版本        :1.0
"""

from pathlib import Path

from gmscaffold.settings import default_settings


class Settings(object):
    """ """

    def __init__(self):
        """ """
        self._settings = default_settings

    def upsert_settings(self):
        """ """
        self.merge_extensions()
        self.reversed_settings()

    def reversed_settings(self):
        """ """

    def merge_extensions(self):
        """ """

        if not self._settings.EXTENSIONS_BASE or not isinstance(self._settings.EXTENSIONS_BASE, dict):
            return

        self._settings.EXTENSIONS = {} if not self._settings.EXTENSIONS else self._settings.EXTENSIONS
        self._settings.EXTENSIONS_BASE |= self._settings.EXTENSIONS

    def __getattr__(self, __name: str):
        """ """
        return getattr(self._settings, __name)

    def extensions(self):
        """Get all extensions info, exclude incomplete data"""
        return dict(
            filter(
                lambda x: bool(x[0]) and bool(x[1]),
                self._settings.EXTENSIONS_TEMPLATE_PATH.items(),
            )
        )

    def extension_template(self, ext_name: str) -> Path | None:
        """Get extension template path

        :param ext_name: Extension name
        :return: Extension template path or None
        """
        ext_dict = self.extensions()
        return ext_dict.get(ext_name)



# -*- encoding: utf-8 -*-
"""
@文件        :cmdline.py
@说明        :
@时间        :2024/11/25 10:54:59
@作者        :Zack
@版本        :1.0
"""
import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
os.environ["PYTHONPATH"] = BASE_DIR
STATIC_DIR = os.path.join(BASE_DIR, "templates")

try:
    pass
except Exception:
    pass


class _BaseExtension:
    """ """

    def __init__(self, module, template, weight=0, version=None):
        """ """
        self.module = module
        self.template = template
        self.weight = weight
        self.version = version  # No features added yet

    def __getitem__(self, key: str):
        """ """
        return self.__dict__[key]

    def __missing__(self, key):
        """ """
        return None


def _construct_extension(name: str, template: str, weight=0):
    """Construct extension info

    :param name: Extension module
    :param template: Extension template path basename
    :param weight: Load extension component weight, No features added yet
    """
    return _BaseExtension(name, os.path.join(STATIC_DIR, template), weight=weight)


SpiderExtension = _construct_extension("gmscaffold.extensions.spider.Spider", "spider")
WebExtension = _construct_extension("gmscaffold.extensions.web.Web", "web")
GmAppExtension = _construct_extension("gmscaffold.extensions.GmApp.GmApp", "GmApp")

EXTENSIONS = [
    SpiderExtension,
    GmAppExtension,
    WebExtension,
]

EXTENSIONS_BASE = {}

EXTENSIONS_TEMPLATE_PATH = {}

for ext_info in EXTENSIONS:
    """ """
    EXTENSIONS_BASE[ext_info["module"]] = ext_info["weight"]
    EXTENSIONS_TEMPLATE_PATH[ext_info["module"]] = ext_info["template"]



# -*- encoding: utf-8 -*-
"""
@文件        :test_cmdline.py
@说明        :
@时间        :2024/11/25 13:40:03
@作者        :Zack
@版本        :1.0
"""
import os
import sys

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(__file__))))

import unittest

from gmscaffold.cmdline import execute


class TestCmdLine(unittest.TestCase):
    """ """

    @unittest.skip
    def test_execute(self):
        """ """
        execute()


if __name__ == "__main__":
    """ """
    unittest.main()



# -*- encoding: utf-8 -*-
"""
@文件        :path.py
@说明        :
@时间        :2024/11/27 15:14:10
@作者        :Zack
@版本        :1.0
"""
import os
from pathlib import Path


def get_files_in_directory(directory: str, ignore_folders=[]):
    """
    递归获取目录中的所有文件。
    :param directory: 要检查的目录。
    :return: 包含文件路径的列表。
    """
    files_list = []
    for entry in os.scandir(directory):
        if entry.is_file():
            files_list.append(entry.path)
        elif entry.is_dir():
            if entry.path in ignore_folders:
                continue
            files_list.extend(get_files_in_directory(entry.path, ignore_folders=ignore_folders))
    return files_list


def replace_path(source: str, old: str, new: str):
    """ """
    return source.replace(old, new)


def replace_suffix(source: str, old: str, new: str) -> str:
    """ """
    s_path = Path(source)
    return os.path.join(s_path.parent, s_path.name.replace(old, new))



# -*- encoding: utf-8 -*-
"""
@文件        :conf.py
@说明        :
@时间        :2024/11/26 14:33:13
@作者        :Zack
@版本        :1.0
"""
import importlib
from typing import Any, Dict, Iterator

from gmscaffold.log import logger  # type: ignore


class ProcessFinder:
    """ """

    def __init__(self, loader):
        """ """
        self._loader = loader

    def find_extensions(self, listdir: Iterator[Dict[str, Any]]):
        """ """
        for module_name in listdir:
            """ """
            module = self._loader.load_module(module_name)
            module.from_settings(self._loader.Gm)


class ProcessLoader:
    """ """

    def __init__(self, Gm):
        """ """
        self.Gm = Gm
        self._finder = ProcessFinder(self)
        self.load_extensions()

    def load_module(self, module_name: str):
        """ """
        module_dirname, basename = module_name.rsplit(".", maxsplit=1)
        module = importlib.__import__(module_dirname, globals(), locals(), fromlist=["*"])
        return getattr(module, basename)

    def load_extensions(self):
        """ """
        self._finder.find_extensions(self.Gm._settings.EXTENSIONS_BASE)



# -*- encoding: utf-8 -*-
"""
@文件        :__main__.py
@说明        :
@时间        :2024/11/25 11:55:11
@作者        :Zack
@版本        :1.0
"""
from gmscaffold.cmdline import execute

if __name__ == "__main__":
    execute()
